@@ -5,7 +5,7 @@ Arithmetic is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -32,7 +32,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 /* Available letters for short options:
 
-   a b d e f i j k l m p r t v x y z
+   b d e f i j k l m p r t v x y z
    A B C E G H J L Q R X Y
 */
 enum option_keys_enum
@@ -46,6 +46,7 @@ enum option_keys_enum
   UI_KEY_METANAME        = 'n',
   UI_KEY_METAUNIT        = 'u',
   UI_KEY_METACOMMENT     = 'c',
+  UI_KEY_WRITEALL        = 'a',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
