@@ -5,7 +5,7 @@ Crop is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -433,6 +433,10 @@ ui_read_check_only_options(struct cropparams *p)
   if(p->mode==IMGCROP_MODE_WCS && p->noblank)
     error(EXIT_FAILURE, 0, "'--noblanks' ('-b') is only for image mode. "
           "You have called it with WCS mode");
+
+  /* If '--apend' has been called, set 'cp.keep' to 1 (since we don't want
+     to delete the output file). */
+  if(p->append) p->cp.keep=1;
 }
 
 
@@ -501,8 +505,8 @@ ui_check_options_and_arguments(struct cropparams *p)
   else
     {
       p->cp.numthreads=1;
-      p->outnameisfile=gal_checkset_dir_0_file_1(p->cp.output,
-                                                 p->cp.dontdelete);
+      p->outnameisfile=gal_checkset_dir_0_file_1(&p->cp, p->cp.output,
+                                                 p->inputs->v);
     }
 }
 
@@ -528,6 +532,7 @@ ui_check_options_and_arguments(struct cropparams *p)
 /**************************************************************/
 /***************       Preparations         *******************/
 /**************************************************************/
+
 /* When the crop is defined by its center, the final width that we need
    must be in actual number of pixels (an integer). But the user's values
    can be in WCS mode or even in image mode, they may be non-integers. */
@@ -574,32 +579,25 @@ ui_set_img_sizes(struct cropparams *p)
   /* WCS mode. */
   if(p->mode==IMGCROP_MODE_WCS)
     {
-      /* Fill in the widths depending on the mode. */
-      for(i=0;i<ndim;++i)
-        {
-          /* Convert the width in units of the input's WCS into pixels. */
-          pwidth = warray[i]/p->pixscale[i];
-          if(pwidth<1 || pwidth>50000)
-            error(EXIT_FAILURE, 0, "value %g (requested width along "
-                  "dimension %zu) translates to %.0f pixels on this "
-                  "dataset. This is probably not what you wanted. Note "
-                  "that the dataset's resolution in this dimension is "
-                  "%g.\n\n"
-                  "You can do the conversion to the dataset's WCS units "
-                  "prior to calling Crop. Alternatively, you can specify "
-                  "all the coordinates/sizes in image (not WCS) units and "
-                  "use the '--mode=img' option", warray[i], i+1, pwidth,
-                  p->pixscale[i]);
+      /* Only convert the width if it is actually in degrees. */
+      if(p->widthinpix==0)
+        for(i=0;i<ndim;++i)
+          {
+            /* Calculate the pixel width. */
+            pwidth = warray[i]/p->pixscale[i];
+            if(pwidth<1 || pwidth>GAL_OPTIONS_WIDTH_TOO_LARGE_SIZE)
+              gal_options_width_too_large(warray[i], i+1, pwidth,
+                                          p->pixscale[i]);
 
-          /* Write the single valued width in WCS and the image width for
-             this dimension. */
-          p->iwidth[i]=GAL_DIMENSION_FLT_TO_INT(pwidth);
-          if(p->iwidth[i]%2==0)
-            {
-              p->iwidth[i] += 1;
-              warray[i]    += p->pixscale[i];
-            }
-        }
+            /* Write the single valued width in WCS and the image width for
+               this dimension. */
+            p->iwidth[i]=GAL_DIMENSION_FLT_TO_INT(pwidth);
+            if(p->iwidth[i]%2==0)
+              {
+                p->iwidth[i] += 1;
+                warray[i]    += p->pixscale[i];
+              }
+          }
 
       /* Checkcenter: */
       if(p->incheckcenter)
@@ -717,9 +715,10 @@ ui_read_cols(struct cropparams *p)
 
 
   /* Read the desired columns from the file. */
-  cols=gal_table_read(p->catname, p->cathdu, NULL, colstrs, p->cp.searchin,
-                      p->cp.ignorecase, p->cp.minmapsize, p->cp.quietmmap,
-                      NULL);
+  cols=gal_table_read(p->catname, p->cathdu, NULL, colstrs,
+                      p->cp.searchin, p->cp.ignorecase,
+                      p->cp.numthreads, p->cp.minmapsize,
+                      p->cp.quietmmap, NULL, "--cathdu");
   if(cols==NULL)
     error(EXIT_FAILURE, 0, "%s: is empty! No usable information "
           "(un-commented lines) could be read from this file",
@@ -733,10 +732,10 @@ ui_read_cols(struct cropparams *p)
   /* Make sure more columns were not read (the name matchings might result
      in more than one column being read from the inputs). */
   if( gal_list_data_number(cols) != ndim + (p->namecol!=NULL) )
-    gal_tableintern_error_col_selection(p->catname, p->cathdu, "too many "
-                                        "columns were selected by the given "
-                                        "values to the options ending in "
-                                        "'col'.");
+    gal_tableintern_error_col_selection(p->catname, p->cathdu, "too "
+                                        "many columns were selected "
+                                        "by the given values to the "
+                                        "options ending in 'col'.");
 
 
   /* Put the information in each column in the proper place. */
@@ -879,10 +878,11 @@ ui_preparations_to_img_mode(struct cropparams *p)
 {
   size_t i;
   int nwcs;
-  double *darr, *pixscale;
+  double *darr, pixwidth, *pixscale;
   struct wcsprm *wcs=gal_wcs_read(p->inputs->v, p->cp.hdu,
                                   p->cp.wcslinearmatrix,
-                                  p->hstartwcs, p->hendwcs, &nwcs);
+                                  p->hstartwcs, p->hendwcs, &nwcs,
+                                  "--hdu");
 
   /* Make sure a WCS actually exists. */
   if(wcs==NULL)
@@ -911,15 +911,27 @@ ui_preparations_to_img_mode(struct cropparams *p)
               "defined by their center (with '--center' or '--catalog') a "
               "width is necessary (using the '--width' option)");
 
-      /* Check the requested width and convert it to pixels. */
-      darr=p->width->array;
+      /* Check if the dataset actually has WCS! */
       if(wcs->naxis<p->width->size)
         error(EXIT_FAILURE, 0, "%s (hdu %s): its WCS has %d dimensions "
               "but %zu dimensions given to '--width'", p->inputs->v,
               p->cp.hdu, wcs->naxis, p->width->size);
-      pixscale=gal_wcs_pixel_scale(wcs);
-      for(i=0;i<p->width->size;++i) darr[i] /= pixscale[i];
-      free(pixscale);
+
+      /* If the width is in WCS units, convert it to pixels. */
+      if(p->widthinpix==0)
+        {
+          darr=p->width->array;
+          pixscale=gal_wcs_pixel_scale(wcs);
+          for(i=0;i<p->width->size;++i)
+            {
+              pixwidth = darr[i] / pixscale[i];
+              if(pixwidth>GAL_OPTIONS_WIDTH_TOO_LARGE_SIZE)
+                gal_options_width_too_large(darr[i], i+1, pixwidth,
+                                            pixscale[i]);
+              darr[i]=pixwidth;
+            }
+          free(pixscale);
+        }
     }
 
   /* Switch the mode and return. */
@@ -1051,7 +1063,7 @@ ui_preparations(struct cropparams *p)
       status=0;
       img=&p->imgs[--input_counter];
       img->name=gal_list_str_pop(&p->inputs);
-      tmpfits=gal_fits_hdu_open_format(img->name, p->cp.hdu, 0);
+      tmpfits=gal_fits_hdu_open_format(img->name, p->cp.hdu, 0, "--hdu");
       gal_fits_img_info(tmpfits, &p->type, &img->ndim, &img->dsize,
                         NULL, NULL);
       img->wcs=gal_wcs_read_fitsptr(tmpfits, p->cp.wcslinearmatrix,
@@ -1276,6 +1288,7 @@ ui_free_report(struct cropparams *p, struct timeval *t1)
   size_t i;
 
   /* Free the simple arrays (if they were set). */
+  free(p->metaname);
   free(p->blankptrread);
   free(p->blankptrwrite);
   gal_data_free(p->center);
