@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2020, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/threads.h>
 #include <gnuastro/pointer.h>
 #include <gnuastro/statistics.h>
+#include <gnuastro/permutation.h>
 #include <gnuastro/interpolate.h>
 
 #include <gnuastro-internal/timing.h>
@@ -162,8 +163,9 @@ threshold_apply(struct noisechiselparams *p, float *value1,
                 float *value2, int kind)
 {
   struct threshold_apply_p taprm={value1, value2, kind, p};
-  gal_threads_spin_off(threshold_apply_on_thread, &taprm, p->cp.tl.tottiles,
-                       p->cp.numthreads);
+  gal_threads_spin_off(threshold_apply_on_thread, &taprm,
+                       p->cp.tl.tottiles, p->cp.numthreads,
+                       p->cp.minmapsize, p->cp.quietmmap);
 }
 
 
@@ -223,7 +225,8 @@ threshold_write_sn_table(struct noisechiselparams *p, gal_data_t *insn,
      because when the output is a FITS table, we want all the tables in one
      FITS file. We have already deleted any existing file with the same
      name in 'ui_set_output_names'.*/
-  gal_table_write(cols, comments, p->cp.tableformat, filename, extname, 0);
+  gal_table_write(cols, NULL, comments, p->cp.tableformat, filename,
+                  extname, 0);
 
 
   /* Clean up (if necessary). */
@@ -458,9 +461,9 @@ qthresh_on_tile(void *in_prm)
               tarray=tile->array; tblock=tile->block;
               tile->array=gal_tile_block_relative_to_other(tile, p->conv);
               tile->block=p->conv;
-              usage->ndim=ndim;             /* Since usage was modified in */
-              usage->size=p->maxtcontig;    /* place, it needs to be       */
-              gal_data_copy_to_allocated(tile, usage);  /* re-initialized. */
+              usage->ndim=ndim;           /* Since usage was modified in */
+              usage->size=p->maxtcontig;  /* place, it needs to be       */
+              gal_data_copy_to_allocated(tile, usage);/* re-initialized. */
               tile->array=tarray; tile->block=tblock;
             }
 
@@ -530,6 +533,8 @@ threshold_good_error(size_t number, int before0_after1, size_t interpnumngb)
                   "(where the number may decrease even further).");
   char *in3 = ( before0_after1
                 ? "\n"
+                  "  - (slightly) Decrease '--outliernumngb' to use less "
+                  "tiles to find outliers.\n"
                   "  - (slightly) Increase '--outliersclip' to reject less "
                   "as outliers.\n"
                   "  - (slightly) Increase '--outliersigma' to reject less "
@@ -607,10 +612,12 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
   /* Allocate space for the quantile threshold values. */
   qprm.erode_th=gal_data_alloc(NULL, p->input->type, p->input->ndim,
                                tl->numtiles, NULL, 0, cp->minmapsize,
-                               p->cp.quietmmap, NULL, p->input->unit, NULL);
+                               p->cp.quietmmap, NULL, p->input->unit,
+                               NULL);
   qprm.noerode_th=gal_data_alloc(NULL, p->input->type, p->input->ndim,
                                  tl->numtiles, NULL, 0, cp->minmapsize,
-                                 p->cp.quietmmap, NULL, p->input->unit, NULL);
+                                 p->cp.quietmmap, NULL, p->input->unit,
+                                 NULL);
   qprm.expand_th = ( p->detgrowquant!=1.0f
                      ? gal_data_alloc(NULL, p->input->type, p->input->ndim,
                                       tl->numtiles, NULL, 0, cp->minmapsize,
@@ -630,7 +637,9 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
      elements, it is only necessary to check one (with the 'updateflag'
      value set to 1), then update the next. */
   qprm.p=p;
-  gal_threads_spin_off(qthresh_on_tile, &qprm, tl->tottiles, cp->numthreads);
+  gal_threads_spin_off(qthresh_on_tile, &qprm, tl->tottiles,
+                       cp->numthreads, cp->minmapsize,
+                       cp->quietmmap);
   free(qprm.usage);
   if( gal_blank_present(qprm.erode_th, 1) )
     {
@@ -662,11 +671,23 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
     }
 
 
-  /* Remove outliers if requested. */
-  if(p->outliersigma!=0.0)
-    gal_tileinternal_no_outlier(qprm.erode_th, qprm.noerode_th,
-                                qprm.expand_th, &p->cp.tl, p->outliersclip,
-                                p->outliersigma, p->qthreshname);
+  /* Remove the outliers. */
+  if(p->outliernumngb)
+    gal_tileinternal_no_outlier_local(qprm.erode_th, qprm.noerode_th,
+                                      qprm.expand_th, &cp->tl,
+                                      cp->interpmetric, p->outliernumngb,
+                                      cp->numthreads, p->outliersclip,
+                                      p->outliersigma, p->qthreshname,
+                                      "--outliernumngb");
+
+
+  /* Use the no-outlier grid as a basis for later estimating the sky. To
+     see this array on the image, use 'gal_tile_full_values_write'. */
+  p->noskytiles=gal_blank_flag(qprm.erode_th);
+  /* For a check:
+  gal_tile_full_values_write(p->noskytiles, &cp->tl, 1,
+                             "noskytiles.fits", NULL, NULL);
+  */
 
 
   /* Check if the number of acceptable tiles is more than the minimum
@@ -677,6 +698,7 @@ threshold_quantile_find_apply(struct noisechiselparams *p)
   nval=((size_t *)(num->array))[0];
   if( nval < cp->interpnumngb )
     threshold_good_error(nval, 1, cp->interpnumngb);
+  gal_data_free(num);
 
 
   /* Interpolate and smooth the derived values. */
