@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2022 Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -75,7 +75,8 @@ segment_convolve(struct segmentparams *p)
           /* Make the convolved image. */
           if(!p->cp.quiet) gettimeofday(&t1, NULL);
           p->conv = gal_convolve_spatial(tl->tiles, p->kernel,
-                                         p->cp.numthreads, 1, tl->workoverch);
+                                         p->cp.numthreads, 1,
+                                         tl->workoverch, 0);
 
           /* Report and write check images if necessary. */
           if(!p->cp.quiet)
@@ -1386,7 +1387,6 @@ segment_output(struct segmentparams *p)
   if(!p->rawoutput)
     gal_fits_img_write(p->input, p->cp.output, NULL, PROGRAM_NAME);
 
-
   /* The clump labels. */
   gal_fits_key_list_add(&keys, GAL_TYPE_FLOAT32, "CLUMPSN", 0,
                         &p->clumpsnthresh, 0, "Minimum S/N of true clumps",
@@ -1399,7 +1399,6 @@ segment_output(struct segmentparams *p)
   p->clabel->name=NULL;
   keys=NULL;
 
-
   /* The object labels. */
   if(!p->noobjects)
     {
@@ -1453,7 +1452,7 @@ segment_output(struct segmentparams *p)
   gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
                               p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "Segment configuration",
-                            "SEGMENT-CONFIG", p->cp.output, "0");
+                            "SEGMENT-CONFIG", p->cp.output, "0", "NONE");
 
   /* Let the user know that the output is written. */
   if(!p->cp.quiet)
