@@ -170,7 +170,8 @@ sky(struct statisticsparams *p)
     {
       if(!cp->quiet) gettimeofday(&t1, NULL);
       p->convolved=gal_convolve_spatial(tl->tiles, p->kernel,
-                                        cp->numthreads, 1, tl->workoverch);
+                                        cp->numthreads, 1,
+                                        tl->workoverch, 0);
       if(p->checksky)
         gal_fits_img_write(p->convolved, p->checkskyname, NULL,
                            PROGRAM_NAME);
@@ -290,7 +291,7 @@ sky(struct statisticsparams *p)
   gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1,
                               p->cp.quiet);
   gal_fits_key_write_config(&p->cp.okeys, "Statistics configuration",
-                            "STATISTICS-CONFIG", outname, "0");
+                            "STATISTICS-CONFIG", outname, "0", "NONE");
   if(!cp->quiet)
     printf("  - Sky and its STD written to '%s'.\n", outname);
 
