@@ -5,7 +5,7 @@ Table is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2022 Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -42,7 +42,8 @@ struct argp_option program_options[] =
       GAL_TYPE_STRLL,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_csv_strings_append
     },
     {
       "wcsfile",
@@ -146,7 +147,7 @@ struct argp_option program_options[] =
       UI_KEY_INFORMATION,
       0,
       0,
-      "Only print table and column information.",
+      "Only print input table's information.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &p->information,
       GAL_OPTIONS_NO_ARG_TYPE,
@@ -154,6 +155,32 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "info-num-rows",
+      UI_KEY_INFONUMROWS,
+      0,
+      0,
+      "Only print input table's number of rows.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->infonumrows,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "info-num-cols",
+      UI_KEY_INFONUMCOLS,
+      0,
+      0,
+      "Only print input table's number of columns.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->infonumcols,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "colinfoinstdout",
       UI_KEY_COLINFOINSTDOUT,
@@ -167,13 +194,113 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "txteasy",
+      UI_KEY_TXTEASY,
+      0,
+      0,
+      "Short for '-Afixed -B6 -ffixed -p3'.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->txteasy,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "txtf32format",
+      UI_KEY_TXTF32FORMAT,
+      "STR",
+      0,
+      "Text output float32 format: 'fixed' or 'exp'.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->txtf32fmtstr,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "txtf64format",
+      UI_KEY_TXTF64FORMAT,
+      "STR",
+      0,
+      "Text output float64 format: 'fixed' or 'exp'.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->txtf64fmtstr,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "txtf32precision",
+      UI_KEY_TXTF32PRECISION,
+      "INT",
+      0,
+      "Text output float32 precision.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->txtf32precision,
+      GAL_TYPE_INT,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "txtf64precision",
+      UI_KEY_TXTF64PRECISION,
+      "INT",
+      0,
+      "Text output float64 precision.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->txtf64precision,
+      GAL_TYPE_INT,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
+
+    /* Operation precendence */
+    {
+      0, 0, 0, 0,
+      "Precedence (default: column operations first)",
+      UI_GROUP_PRECEDENCE
+    },
+    {
+      "rowfirst",
+      UI_KEY_ROWFIRST,
+      0,
+      0,
+      "Apply row-based operations first.",
+      UI_GROUP_PRECEDENCE,
+      &p->rowfirst,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
+
+    /* Output columns. */
+    {
+      0, 0, 0, 0,
+      "Columns in output:",
+      UI_GROUP_OUTCOLS
+    },
     {
       "catcolumnrawname",
       UI_KEY_CATCOLUMNRAWNAME,
       0,
       0,
       "Don't touch column names of --catcolumnfile.",
-      GAL_OPTIONS_GROUP_OUTPUT,
+      UI_GROUP_OUTCOLS,
       &p->catcolumnrawname,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -183,10 +310,10 @@ struct argp_option program_options[] =
     {
       "colmetadata",
       UI_KEY_COLMETADATA,
-      "STR,STR[,STR,STR]",
+      "ID,S,S,S",
       0,
-      "Update output metadata (name, unit, comments).",
-      GAL_OPTIONS_GROUP_OUTPUT,
+      "Column metadata (S=STR: Name, Unit, Comments).",
+      UI_GROUP_OUTCOLS,
       &p->colmetadata,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -194,7 +321,59 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_parse_name_and_strings
     },
-
+    {
+      "tovector",
+      UI_KEY_TOVECTOR,
+      "STR,STR[,STR]",
+      0,
+      "Merge column(s) into a vector column.",
+      UI_GROUP_OUTCOLS,
+      &p->tovector,
+      GAL_TYPE_STRLL,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "fromvector",
+      UI_KEY_FROMVECTOR,
+      "STR,INT[,INT]",
+      0,
+      "Extract column(s) from a vector column.",
+      UI_GROUP_OUTCOLS,
+      &p->fromvector,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_parse_name_and_sizets
+    },
+    {
+      "keepvectfin",
+      UI_KEY_KEEPVECTFIN,
+      0,
+      0,
+      "Keep inputs of '--tovector' & '--fromvector'.",
+      UI_GROUP_OUTCOLS,
+      &p->keepvectfin,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "transpose",
+      UI_KEY_TRANSPOSE,
+      0,
+      0,
+      "Transpose table (must only contain vector cols).",
+      UI_GROUP_OUTCOLS,
+      &p->transpose,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -342,13 +521,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "rowlimit",
-      UI_KEY_ROWLIMIT,
+      "rowrange",
+      UI_KEY_ROWRANGE,
       "INT,INT",
       0,
       "Only rows in this row-counter range.",
       UI_GROUP_OUTROWS,
-      &p->rowlimit,
+      &p->rowrange,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_NOT_MANDATORY,
