@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2021, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -92,18 +92,18 @@ convolve_tile_is_on_edge(size_t *h, size_t *start_end_coord, size_t *k,
 struct per_thread_spatial_prm
 {
   /* Internally stored/used values. */
-  size_t             id;     /* ID of tile being operatred on.           */
-  gal_data_t      *tile;     /* Tile this thread is working on.          */
-  gal_data_t *i_overlap;     /* Overlap tile over input dataset.         */
-  gal_data_t *k_overlap;     /* Overlap tile over kernel dataset.        */
-  size_t *overlap_start;     /* Starting coordinate of kernel overlap.   */
-  size_t  *kernel_start;     /* Kernel starting point.                   */
-  size_t    *host_start;     /* Starting coordinate of host.             */
-  size_t           *pix;     /* 2*ndim: starting and ending of tile,
-                                Later, just the pixel being convolved.   */
-  int           on_edge;     /* If the tile is on the edge or not.       */
-  gal_data_t      *host;     /* Size of host (channel or block).         */
-  struct spatial_params *cprm; /* Link to main structure for all threads.*/
+  size_t             id;     /* ID of tile being operatred on.            */
+  gal_data_t      *tile;     /* Tile this thread is working on.           */
+  gal_data_t *i_overlap;     /* Overlap tile over input dataset.          */
+  gal_data_t *k_overlap;     /* Overlap tile over kernel dataset.         */
+  size_t *overlap_start;     /* Starting coordinate of kernel overlap.    */
+  size_t  *kernel_start;     /* Kernel starting point.                    */
+  size_t    *host_start;     /* Starting coordinate of host.              */
+  size_t           *pix;     /* 2*ndim: starting and ending of tile,      */
+                             /* Later, just the pixel being convolved.    */
+  int           on_edge;     /* If the tile is on the edge or not.        */
+  gal_data_t      *host;     /* Size of host (channel or block).          */
+  struct spatial_params *cprm; /* Link to main structure for all threads. */
 };
 
 
@@ -113,14 +113,15 @@ struct per_thread_spatial_prm
 struct spatial_params
 {
   /* Main input/output parameters. */
-  gal_data_t       *out;     /* Output data structure.                   */
-  gal_data_t     *tiles;     /* Tiles over the input image.              */
-  gal_data_t     *block;     /* Pointer to block for this tile.          */
-  gal_data_t    *kernel;     /* Kernel to convolve with input.           */
-  gal_data_t *tocorrect;     /* (possible) convolved image to correct.   */
-  int        convoverch;     /* Ignore channel edges in convolution.     */
-  int    edgecorrection;     /* Correct convolution's edge effects.      */
-  struct per_thread_spatial_prm *pprm; /* Array of per-thread parameters.*/
+  gal_data_t       *out;     /* Output data structure.                    */
+  gal_data_t     *tiles;     /* Tiles over the input image.               */
+  gal_data_t     *block;     /* Pointer to block for this tile.           */
+  gal_data_t    *kernel;     /* Kernel to convolve with input.            */
+  gal_data_t *tocorrect;     /* (possible) convolved image to correct.    */
+  int        convoverch;     /* Ignore channel edges in convolution.      */
+  int    edgecorrection;     /* Correct convolution's edge effects.       */
+  uint8_t conv_on_blank;     /* Do convolution over blank pixels also.    */
+  struct per_thread_spatial_prm *pprm; /* Array of per-thread parameters. */
 };
 
 
@@ -129,7 +130,7 @@ struct spatial_params
 
 /* Define the overlap of the kernel and image over this part of the image,
    the necessary input image parameters are stored in 'overlap' (its
-   'array' and 'dsize' elements).  */
+   'array' and 'dsize' elements). */
 static int
 convolve_spatial_overlap(struct per_thread_spatial_prm *pprm, int tocorrect)
 {
@@ -226,7 +227,7 @@ convolve_spatial_overlap(struct per_thread_spatial_prm *pprm, int tocorrect)
         }
 
       /* There is full overlap for this pixel or tile over this
-         dimension.  */
+         dimension. */
       if(dim_full_overlap)
         {
           /* Set the values. */
@@ -358,7 +359,7 @@ convolve_spatial_tile(struct per_thread_spatial_prm *pprm)
           /* If the input on this pixel is a NaN, then just set the output
              to NaN too and go onto the next pixel. 'in_v' is the pointer
              on this pixel. */
-          if( isnan(*in_v) )
+          if( isnan(*in_v) && cprm->conv_on_blank==0 )
             out[ in_v - in ]=NAN;
           else
             {
@@ -483,11 +484,12 @@ convolve_spatial_on_thread(void *inparam)
 
 
 /* General spatial convolve function. This function is called by both
-   'gal_convolve_spatial' and */
+   'gal_convolve_spatial' and 'gal_convolve_spatial_correct_ch_edge'. */
 static gal_data_t *
 gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
                              size_t numthreads, int edgecorrection,
-                             int convoverch, gal_data_t *tocorrect)
+                             int convoverch, uint8_t conv_on_blank,
+                             gal_data_t *tocorrect)
 {
   struct spatial_params params;
   gal_data_t *out, *block=gal_tile_block(tiles);
@@ -504,7 +506,7 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
   /* It may happen that an input dataset is part of a linked list, but it
      is not actually a tile structure (the user wants to convolve the whole
      dataset without using tiles)! In that case, this function should break
-     beacuse a linked list is interpretted as a tile structure here.*/
+     beacuse a linked list is interpretted as a tile structure here. */
   if( tiles->block==NULL && tiles->next && tiles->next->block==NULL )
     error(EXIT_FAILURE, 0, "%s: the input is a linked list but not a "
           "tessellation (a list of tiles). This function is optimized to "
@@ -513,7 +515,7 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
           __func__);
 
 
-  /* Set the output datastructure.  */
+  /* Set the output datastructure. */
   if(tocorrect) out=tocorrect;
   else
     {
@@ -536,6 +538,7 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
   params.kernel=kernel;
   params.tocorrect=tocorrect;
   params.convoverch=convoverch;
+  params.conv_on_blank=conv_on_blank;
   params.edgecorrection=edgecorrection;
 
 
@@ -566,10 +569,11 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
    convolution can be greatly sped up if it is done on separate tiles over
    the image (on multiple threads). So as input, you can either give tile
    values or one full array. Just note that if you give a single array as
-   input, the 'next' element has to be 'NULL'.*/
+   input, the 'next' element has to be 'NULL'. */
 gal_data_t *
 gal_convolve_spatial(gal_data_t *tiles, gal_data_t *kernel,
-                     size_t numthreads, int edgecorrection, int convoverch)
+                     size_t numthreads, int edgecorrection, int convoverch,
+                     int conv_on_blank)
 {
   /* When there isn't any tile structure, 'convoverch' must be set to
      one. Recall that the input can be a single full dataset also. */
@@ -577,7 +581,8 @@ gal_convolve_spatial(gal_data_t *tiles, gal_data_t *kernel,
 
   /* Call the general function. */
   return gal_convolve_spatial_general(tiles, kernel, numthreads,
-                                      edgecorrection, convoverch, NULL);
+                                      edgecorrection, convoverch,
+                                      conv_on_blank, NULL);
 }
 
 
@@ -593,7 +598,7 @@ gal_convolve_spatial(gal_data_t *tiles, gal_data_t *kernel,
 void
 gal_convolve_spatial_correct_ch_edge(gal_data_t *tiles, gal_data_t *kernel,
                                      size_t numthreads, int edgecorrection,
-                                     gal_data_t *tocorrect)
+                                     int conv_on_blank, gal_data_t *tocorrect)
 {
   gal_data_t *block=gal_tile_block(tiles);
 
@@ -609,5 +614,6 @@ gal_convolve_spatial_correct_ch_edge(gal_data_t *tiles, gal_data_t *kernel,
 
   /* Call the general function, which will do the correction. */
   gal_convolve_spatial_general(tiles, kernel, numthreads,
-                               edgecorrection, 0, tocorrect);
+                               edgecorrection, 0, conv_on_blank,
+                               tocorrect);
 }
