@@ -6,7 +6,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2022 Free Software Foundation, Inc.
+Copyright (C) 2018-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -72,7 +72,8 @@ gal_tileinternal_no_outlier_local(gal_data_t *input, gal_data_t *second,
                                   struct gal_tile_two_layer_params *tl,
                                   uint8_t metric, size_t numneighbors,
                                   size_t numthreads, double *outliersclip,
-                                  double outliersigma, char *filename);
+                                  double outliersigma, char *filename,
+                                  char *optionname);
 
 __END_C_DECLS    /* From C++ preparations */
 
