@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2020, Free Software Foundation, Inc.
+Copyright (C) 2017-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -27,6 +27,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
    must be included before the C++ preparations below */
 #include <gnuastro/data.h>
 #include <gnuastro/blank.h>
+#include <gnuastro/polygon.h>
 
 /* C++ Preparations */
 #undef __BEGIN_C_DECLS
@@ -72,6 +73,28 @@ gal_dimension_num_neighbors(size_t ndim);
 #define GAL_DIMENSION_FLT_TO_INT(FLT) ( (FLT)-(long)(FLT) > 0.5f  \
                                         ? (long)(FLT)+1 : (long)(FLT) )
 
+/* A pixel's center is an integer value. This function will give the
+   integer value that is nearest to a floating point number. Works for both
+   positive and negative values and includes floating point errors.
+
+   WARP_NEARESTINT_HALFHIGHER(0.5f) --> 1.0f
+*/
+#define GAL_DIMENSION_NEARESTINT_HALFHIGHER(D)         \
+  ( ceil( (D) ) - (D) > 0.5f + GAL_POLYGON_ROUND_ERR ? \
+                            ceil((D))-1.0f : ceil((D)) )
+
+/* Similar to 'GAL_DIMENSION_NEARESTINT_HALFHIGHER' but:
+
+   GAL_DIMENSION_NEARESTINT_HALFLOWER(0.5f) --> 0.0f; */
+#define GAL_DIMENSION_NEARESTINT_HALFLOWER(D)            \
+  ( ceil( (D) ) - (D) > 0.5f - GAL_POLYGON_ROUND_ERR ?   \
+                            ceil((D))-1.0f : ceil((D)) )
+
+#define GAL_DIMENSION_CEILWITHERR(D)                       \
+  ( fabs( nearbyint((D)) - (D) ) < GAL_POLYGON_ROUND_ERR ? \
+                                   nearbyint((D)) : nearbyint((D))+1    )
+
+
 void
 gal_dimension_add_coords(size_t *c1, size_t *c2, size_t *out, size_t ndim);
 
@@ -95,6 +118,10 @@ gal_dimension_dist_manhattan(size_t *a, size_t *b, size_t ndim);
 float
 gal_dimension_dist_radial(size_t *a, size_t *b, size_t ndim);
 
+float
+gal_dimension_dist_elliptical(double *center, double *pa_deg, double *q,
+                              size_t ndim, double *point);
+
 
 
 /************************************************************************/
@@ -113,6 +140,35 @@ gal_dimension_collapse_number(gal_data_t *in, size_t c_dim);
 gal_data_t *
 gal_dimension_collapse_minmax(gal_data_t *in, size_t c_dim, int max1_min0);
 
+gal_data_t *
+gal_dimension_collapse_median(gal_data_t *in, size_t c_dim,
+                              size_t numthreads, size_t minmapsize,
+                              int quietmmap);
+
+gal_data_t *
+gal_dimension_collapse_sclip_std(gal_data_t *in, size_t c_dim,
+                                 float multip, float param,
+                                 size_t numthreads, size_t minmapsize,
+                                 int quietmmap);
+
+gal_data_t *
+gal_dimension_collapse_sclip_mean(gal_data_t *in, size_t c_dim,
+                                  float multip, float param,
+                                  size_t numthreads, size_t minmapsize,
+                                  int quietmmap);
+
+gal_data_t *
+gal_dimension_collapse_sclip_median(gal_data_t *in, size_t c_dim,
+                                    float multip, float param,
+                                    size_t numthreads, size_t minmapsize,
+                                    int quietmmap);
+
+gal_data_t *
+gal_dimension_collapse_sclip_number(gal_data_t *in, size_t c_dim,
+                                    float multip, float param,
+                                    size_t numthreads, size_t minmapsize,
+                                    int quietmmap);
+
 
 
 /************************************************************************/
