@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2021, Free Software Foundation, Inc.
+Copyright (C) 2015-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -92,6 +92,63 @@ enum gal_wcs_linear_matrix
   GAL_WCS_LINEAR_MATRIX_CD,
 };
 
+/* Macros for various projections (these names come from the 'prj.h' file
+   of WCSLIB (in '/usr/local/include/wcslib' for version 8.1). */
+enum gal_wcs_projections
+{
+  GAL_WCS_PROJECTION_INVALID,   /* Invalid (=0 by C standard). */
+  GAL_WCS_PROJECTION_AZP,       /* zenithal/azimuthal perspective */
+  GAL_WCS_PROJECTION_SZP,       /* slant zenithal perspective */
+  GAL_WCS_PROJECTION_TAN,       /* gnomonic */
+  GAL_WCS_PROJECTION_STG,       /* stereographic */
+  GAL_WCS_PROJECTION_SIN,       /* orthographic/synthesis */
+  GAL_WCS_PROJECTION_ARC,       /* zenithal/azimuthal equidistant */
+  GAL_WCS_PROJECTION_ZPN,       /* zenithal/azimuthal polynomial */
+  GAL_WCS_PROJECTION_ZEA,       /* zenithal/azimuthal equal area */
+  GAL_WCS_PROJECTION_AIR,       /* Airy */
+  GAL_WCS_PROJECTION_CYP,       /* cylindrical perspective */
+  GAL_WCS_PROJECTION_CEA,       /* cylindrical equal area */
+  GAL_WCS_PROJECTION_CAR,       /* Plate carree */
+  GAL_WCS_PROJECTION_MER,       /* Mercator */
+  GAL_WCS_PROJECTION_SFL,       /* Sanson-Flamsteed */
+  GAL_WCS_PROJECTION_PAR,       /* parabolic */
+  GAL_WCS_PROJECTION_MOL,       /* Mollweide */
+  GAL_WCS_PROJECTION_AIT,       /* Hammer-Aitoff */
+  GAL_WCS_PROJECTION_COP,       /* conic perspective */
+  GAL_WCS_PROJECTION_COE,       /* conic equal area */
+  GAL_WCS_PROJECTION_COD,       /* conic equidistant */
+  GAL_WCS_PROJECTION_COO,       /* conic orthomorphic */
+  GAL_WCS_PROJECTION_BON,       /* Bonne */
+  GAL_WCS_PROJECTION_PCO,       /* polyconic */
+  GAL_WCS_PROJECTION_TSC,       /* tangential spherical cube */
+  GAL_WCS_PROJECTION_CSC,       /* COBE spherical cube */
+  GAL_WCS_PROJECTION_QSC,       /* quadrilateralized spherical cube */
+  GAL_WCS_PROJECTION_HPX,       /* HEALPix */
+  GAL_WCS_PROJECTION_XPH,       /* HEALPix polar, aka "butterfly" */
+};
+
+
+
+
+
+/*************************************************************
+ ***********               Macros                  ***********
+ *************************************************************/
+int
+gal_wcs_distortion_name_to_id(char *distortion);
+
+char *
+gal_wcs_distortion_name_from_id(int distortion);
+
+int
+gal_wcs_coordsys_name_to_id(char *coordsys);
+
+uint8_t
+gal_wcs_projection_name_to_id(char *str);
+
+char *
+gal_wcs_projection_name_from_id(uint8_t id);
+
 
 
 
@@ -104,14 +161,18 @@ gal_wcs_read_fitsptr(fitsfile *fptr, int linearmatrix, size_t hstartwcs,
                      size_t hendwcs, int *nwcs);
 
 struct wcsprm *
-gal_wcs_read(char *filename, char *hdu, int linearmatrix, size_t hstartwcs,
-             size_t hendwcs, int *nwcs);
+gal_wcs_read(char *filename, char *hdu, int linearmatrix,
+             size_t hstartwcs, size_t hendwcs, int *nwcs,
+             char *hdu_option_name);
 
 struct wcsprm *
 gal_wcs_create(double *crpix, double *crval, double *cdelt,
                double *pc, char **cunit, char **ctype, size_t ndim,
                int linearmatrix);
 
+void
+gal_wcs_free(struct wcsprm *wcs);
+
 char *
 gal_wcs_dimension_name(struct wcsprm *wcs, size_t dimension);
 
@@ -136,26 +197,24 @@ gal_wcs_write_in_fitsptr(fitsfile *fptr, struct wcsprm *wcs);
 /*************************************************************
  ***********              Distortions              ***********
  *************************************************************/
-int
-gal_wcs_coordsys_from_string(char *coordsys);
-
 int
 gal_wcs_coordsys_identify(struct wcsprm *inwcs);
 
 struct wcsprm *
 gal_wcs_coordsys_convert(struct wcsprm *inwcs, int coordsysid);
 
+void
+gal_wcs_coordsys_convert_points(int sys1, double *lng1_d, double *lat1_d,
+                                int sys2, double *lng2_d, double *lat2_d,
+                                size_t number);
 
+void
+gal_wcs_coordsys_sys1_ref_in_sys2(int sys1, int sys2, double *lng2,
+                                  double *lat2);
 
 /*************************************************************
  ***********              Distortions              ***********
  *************************************************************/
-int
-gal_wcs_distortion_from_string(char *distortion);
-
-char *
-gal_wcs_distortion_to_string(int distortion);
-
 int
 gal_wcs_distortion_identify(struct wcsprm *wcs);
 
@@ -173,6 +232,9 @@ gal_wcs_distortion_convert(struct wcsprm *inwcs,
 struct wcsprm *
 gal_wcs_copy(struct wcsprm *wcs);
 
+struct wcsprm *
+gal_wcs_copy_new_crval(struct wcsprm *in, double *crval);
+
 void
 gal_wcs_remove_dimension(struct wcsprm *wcs, size_t fitsdim);
 
@@ -194,6 +256,11 @@ gal_wcs_to_cd(struct wcsprm *wcs);
 double
 gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2);
 
+void
+gal_wcs_box_vertices_from_center(double ra_center, double dec_center,
+                                 double ra_delta,  double dec_delta,
+                                 double *out);
+
 double *
 gal_wcs_pixel_scale(struct wcsprm *wcs);
 
@@ -201,9 +268,11 @@ double
 gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs);
 
 int
-gal_wcs_coverage(char *filename, char *hdu, size_t *ndim,
-                 double **center, double **width, double **min,
-                 double **max);
+gal_wcs_coverage(char *filename, char *hdu, size_t *ondim,
+                 double **ocenter, double **owidth, double **omin,
+                 double **omax, char *hdu_option_name);
+
+
 
 
 
