@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2021, Free Software Foundation, Inc.
+Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -33,6 +33,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/txt.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/table.h>
+#include <gnuastro/pointer.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/checkset.h>
@@ -43,6 +44,49 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 
+/************************************************************************/
+/***************            Internal conversions          ***************/
+/************************************************************************/
+uint8_t
+gal_table_displayflt_from_str(char *string)
+{
+  if(      !strcmp(string, "exp"))
+    return GAL_TABLE_DISPLAY_FMT_EXP;
+  else if (!strcmp(string, "fixed"))
+    return GAL_TABLE_DISPLAY_FMT_FIXED;
+  else return GAL_TABLE_DISPLAY_FMT_INVALID;
+}
+
+
+
+
+
+char *
+gal_table_displayflt_to_str(uint8_t fmt)
+{
+  switch(fmt)
+    {
+    case GAL_TABLE_DISPLAY_FMT_FIXED: return "fixed";
+    case GAL_TABLE_DISPLAY_FMT_EXP:   return "exp";
+    default:                          return NULL;
+    }
+}
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 
@@ -52,20 +96,19 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Store the information of each column in a table (either as a text file
    or as a FITS table) into an array of data structures with 'numcols'
    structures (one data structure for each column). The number of rows is
-   stored in 'numrows'. The type of the table (e.g., ascii text file, or
-   FITS binary or ASCII table) will be put in 'tableformat' (macros defined
-   in 'gnuastro/table.h'.
+   stored in 'numrows'.\
 
-   Note that other than the character strings (column name, units and
-   comments), nothing in the data structure(s) will be allocated by this
-   function for the actual data (e.g., the 'array' or 'dsize' elements). */
+   See the DESCRIPTION OF THIS FUNCTION IN THE BOOK FOR a detailed listing
+   of the output's elements. */
 gal_data_t *
 gal_table_info(char *filename, char *hdu, gal_list_str_t *lines,
-               size_t *numcols, size_t *numrows, int *tableformat)
+               size_t *numcols, size_t *numrows, int *tableformat,
+               char *hdu_option_name)
 {
   /* Get the table format and size (number of columns and rows). */
   if(filename && gal_fits_file_recognized(filename))
-    return gal_fits_tab_info(filename, hdu, numcols, numrows, tableformat);
+    return gal_fits_tab_info(filename, hdu, numcols, numrows, tableformat,
+                             hdu_option_name);
   else
     {
       *tableformat=GAL_TABLE_FORMAT_TXT;
@@ -73,9 +116,9 @@ gal_table_info(char *filename, char *hdu, gal_list_str_t *lines,
     }
 
   /* Abort with an error if we get to this point. */
-  error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we can fix "
-        "the problem. Control must not have reached the end of this function",
-        __func__, PACKAGE_BUGREPORT);
+  error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we can "
+        "fix the problem. Control must not have reached the end of this "
+        "function", __func__, PACKAGE_BUGREPORT);
   return NULL;
 }
 
@@ -86,9 +129,9 @@ gal_table_info(char *filename, char *hdu, gal_list_str_t *lines,
 void
 gal_table_print_info(gal_data_t *allcols, size_t numcols, size_t numrows)
 {
-  size_t i;
-  int Nw=3, nw=4, uw=5, tw=4;   /* Initial width from label's width */
-  char *name, *unit, *comment;
+  size_t i, mms;
+  int Nw=3, nw=4, uw=5, tw=4, twt;/* Initial width from label's width */
+  char *typestr, *name, *unit, *comment;
 
   /* If there aren't any columns, there is no need to print anything. */
   if(numcols==0) return;
@@ -99,13 +142,23 @@ gal_table_print_info(gal_data_t *allcols, size_t numcols, size_t numrows)
   Nw=log10(numcols)+1;
   for(i=0;i<numcols;++i)
     {
+      /* Length of name and unit columns. */
       if(allcols[i].name && strlen(allcols[i].name)>nw)
         nw=strlen(allcols[i].name);
       if(allcols[i].unit && strlen(allcols[i].unit)>uw)
         uw=strlen(allcols[i].unit);
-      if(allcols[i].type
-         && strlen(gal_type_name(allcols[i].type, 1))>tw)
-        tw=strlen(gal_type_name(allcols[i].type, 1));
+
+      /* For the type, we need to account for vector columns. For strings,
+         their length is already accounted for there, so this doesn't
+         involve them. Recall that within 'gal_fits_tab_info', we put the
+         repeat counter into the 'minmapsize' element of the
+         'gal_data_t'. If the repeat of a numerical column is more than 1,
+         then we need to add a '[N]' after the type name later. */
+      mms=allcols[i].minmapsize;
+      twt=strlen(gal_type_name(allcols[i].type, 1));
+      if(allcols[i].type!=GAL_TYPE_STRING && mms>1)
+        twt += (int)(log10(mms))+1+2; /* 1 for the log, 2 for '()'. */
+      if(allcols[i].type && twt>tw) tw=twt;
     }
 
   /* We want one column space between the columns for readability, not the
@@ -123,15 +176,31 @@ gal_table_print_info(gal_data_t *allcols, size_t numcols, size_t numrows)
   /* For each column, print the information, then free them. */
   for(i=0;i<numcols;++i)
     {
+      /* To help in reading. */
       name    = allcols[i].name;       /* Just defined for easier     */
       unit    = allcols[i].unit;       /* readability. The compiler   */
       comment = allcols[i].comment;    /* optimizer will remove them. */
+
+      /* Set the vector size (if relevant). */
+      mms=allcols[i].minmapsize;
+      if(allcols[i].type!=GAL_TYPE_STRING && mms>1)
+        {
+          if( asprintf(&typestr, "%s(%zu)",
+                       gal_type_name(allcols[i].type, 1), mms)<0 )
+            error(EXIT_FAILURE, 0, "%s: 'astprintf' allocation", __func__);
+        }
+      else
+        gal_checkset_allocate_copy(gal_type_name(allcols[i].type, 1),
+                                   &typestr);
+
+      /* Print the actual info. */
       printf("%-*zu%-*s%-*s%-*s%s\n", Nw, i+1,
              nw, name ? name : GAL_BLANK_STRING ,
              uw, unit ? unit : GAL_BLANK_STRING ,
              tw,
-             allcols[i].type ? gal_type_name(allcols[i].type, 1) : "--",
+             allcols[i].type ? typestr : "--",
              comment ? comment : GAL_BLANK_STRING);
+      free(typestr);
     }
 
   /* Print the number of rows. */
@@ -250,8 +319,8 @@ gal_table_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
             errno=0;
             regex=malloc(sizeof *regex);
             if(regex==NULL)
-              error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for regex",
-                    __func__, sizeof *regex);
+              error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for "
+                    "regex", __func__, sizeof *regex);
 
             /* First we have to "compile" the string into the regular
                expression, see the "POSIX Regular Expression Compilation"
@@ -415,7 +484,8 @@ gal_table_list_of_indexs(gal_list_str_t *cols, gal_data_t *allcols,
 gal_data_t *
 gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
                gal_list_str_t *cols, int searchin, int ignorecase,
-               size_t minmapsize, int quietmmap, size_t *colmatch)
+               size_t numthreads, size_t minmapsize, int quietmmap,
+               size_t *colmatch, char *hdu_option_name)
 {
   int tableformat;
   gal_list_sizet_t *indexll;
@@ -424,7 +494,7 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
 
   /* First get the information of all the columns. */
   allcols=gal_table_info(filename, hdu, lines, &numcols, &numrows,
-                         &tableformat);
+                         &tableformat, hdu_option_name);
 
   /* If there was no actual data in the file, then return NULL. */
   if(allcols==NULL) return NULL;
@@ -434,15 +504,8 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
                                    ignorecase, filename, hdu, colmatch);
 
   /* Depending on the table format, read the columns into the output
-     structure. Note that the functions here pop each index, read/store the
-     desired column and pop the next, so after these functions, the output
-     linked list will have the opposite order of its input 'indexll'
-     list. So before calling any of them, we will first reverse the
-     'indexll' list, so the output data structure list will have the same
-     order as the input list of desired columns. Also note that after these
-     functions, the 'indexll' will be all freed (each popped element is
-     actually freed).*/
-  gal_list_sizet_reverse(&indexll);
+     structure. Also note that after these functions, the 'indexll' will be
+     all freed (each popped element is actually freed).*/
   switch(tableformat)
     {
     case GAL_TABLE_FORMAT_TXT:
@@ -453,7 +516,8 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
     case GAL_TABLE_FORMAT_AFITS:
     case GAL_TABLE_FORMAT_BFITS:
       out=gal_fits_tab_read(filename, hdu, numrows, allcols, indexll,
-                            minmapsize, quietmmap);
+                            numthreads, minmapsize, quietmmap,
+                            hdu_option_name);
       break;
 
     default:
@@ -497,8 +561,8 @@ gal_table_read(char *filename, char *hdu, gal_list_str_t *lines,
    comments field. Note that the 'comments' has to be already sorted in the
    proper order. */
 void
-gal_table_comments_add_intro(gal_list_str_t **comments, char *program_string,
-                             time_t *rawtime)
+gal_table_comments_add_intro(gal_list_str_t **comments,
+                             char *program_string, time_t *rawtime)
 {
   char gitdescribe[100], *tmp;
 
@@ -544,11 +608,12 @@ gal_table_write(gal_data_t *cols, struct gal_fits_list_key_t **keylist,
         gal_fits_tab_write(cols, comments, tableformat, filename, extname,
                            keylist);
       else
-        gal_txt_write(cols, keylist, comments, filename, colinfoinstdout);
+        gal_txt_write(cols, keylist, comments, filename,
+                      colinfoinstdout, 0);
     }
   else
     /* Write to standard output. */
-    gal_txt_write(cols, keylist, comments, filename, colinfoinstdout);
+    gal_txt_write(cols, keylist, comments, filename, colinfoinstdout, 0);
 }
 
 
@@ -578,3 +643,137 @@ gal_table_write_log(gal_data_t *logll, char *program_string,
       free(msg);
     }
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/************************************************************************/
+/***************            Column operation              ***************/
+/************************************************************************/
+gal_data_t *
+gal_table_col_vector_extract(gal_data_t *vector, gal_list_sizet_t *indexs)
+{
+  uint8_t type;
+  gal_data_t *out=NULL;
+  size_t i, vw, vh, ind;
+  char *name, *basename;
+  gal_list_sizet_t *tind;
+
+  /* Basic sanity checks. */
+  if(vector==NULL) return NULL;
+  if(indexs==NULL) return NULL;
+  if(vector->ndim!=2)
+    error(EXIT_FAILURE, 0, "%s: the input 'vector' must have 2 "
+          "dimensions but has %zu dimensions", __func__, vector->ndim);
+  for(tind=indexs;tind!=NULL;tind=tind->next)
+    if(tind->v > vector->dsize[1])
+      error(EXIT_FAILURE, 0, "%s: the input vector has %zu elements but "
+            "you have asked for index %zu (counting from zero)", __func__,
+            vector->dsize[1], tind->v);
+
+  /* Allocate the output columns. */
+  type=vector->type;
+  vh=vector->dsize[0];
+  vw=vector->dsize[1];
+  for(tind=indexs;tind!=NULL;tind=tind->next)
+    {
+      /* If the vector has a name, add a counter after it. If it doesn't
+         have a name, just use 'VECTOR'. Because index couting starts from
+         0, but column counters in tables start with 1, we'll add one to
+         the index. */
+      ind=tind->v;
+      basename = vector->name ? vector->name : "VECTOR";
+      if( asprintf(&name, "%s-%zu", basename, ind+1)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf alloc of 'name'",  __func__);
+
+      /* Allocate the output and fill it. */
+      gal_list_data_add_alloc(&out, NULL, type, 1, &vh, NULL, 1,
+                              vector->minmapsize, vector->quietmmap,
+                              name, vector->unit, vector->comment);
+      for(i=0;i<vh;++i)
+        memcpy(gal_pointer_increment(out->array,    i,        type),
+               gal_pointer_increment(vector->array, i*vw+ind, type),
+               gal_type_sizeof(type));
+
+      /* Clean up. */
+      free(name);
+    }
+
+  /* Reverse the list to be in the same order as the indexs, and return. */
+  gal_list_data_reverse(&out);
+  return out;
+}
+
+
+
+
+
+gal_data_t *
+gal_table_cols_to_vector(gal_data_t *list)
+{
+  gal_data_t *tmp, *out;
+  char *name, *unit=NULL, *inname=NULL;
+  size_t i, j, dsize[2], num=gal_list_data_number(list);
+
+  /* If the list is empty or just has a single column, return itself.*/
+  if(num<2) return list;
+
+  /* Go over the inputs na make sure they are all single dimensional and
+     have the same size. */
+  for(tmp=list;tmp!=NULL;tmp=tmp->next)
+    {
+      /* First, a sanity check. */
+      if(tmp->ndim!=1
+         || tmp->type!=list->type
+         || tmp->dsize[0]!=list->dsize[0])
+        error(EXIT_FAILURE, 0, "%s: inputs should all be single-valued "
+              "columns (one dimension) and have the same size and type",
+              __func__);
+
+      /* Find the first one with a name. */
+      if(tmp->unit && unit==NULL)   unit=tmp->unit;
+      if(tmp->name && inname==NULL) inname=tmp->name;
+    }
+
+  /* Set the name based on the input. */
+  if( asprintf(&name, "%s-VECTOR", inname?inname:"TO")<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+
+  /* Allocate the output dataset. */
+  dsize[1]=num;
+  dsize[0]=list->size;
+  out=gal_data_alloc(NULL, list->type, 2, dsize, NULL, 0, list->minmapsize,
+                     list->quietmmap, name, unit,
+                     "Vector by merging multiple columns.");
+
+  /* Fill the output dataset. */
+  j=0;
+  for(tmp=list;tmp!=NULL;tmp=tmp->next)
+    {
+      for(i=0;i<tmp->dsize[0];++i)
+        memcpy(gal_pointer_increment(out->array, i*num+j, out->type),
+               gal_pointer_increment(tmp->array, i,       out->type),
+               gal_type_sizeof(out->type));
+      ++j;
+    }
+
+  /* Clean up and return. */
+  free(name);
+  return out;
+}
