@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018-2022 Free Software Foundation, Inc.
+     Fathma Mehnoor <fathmamehnoor@gmail.com>
+Copyright (C) 2018-2023 Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -37,6 +38,8 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/tiff.h>
 #include <gnuastro/pointer.h>
 
+#include <gnuastro-internal/checkset.h>
+
 
 
 
@@ -50,7 +53,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
  *************************************************************/
 #ifndef HAVE_LIBTIFF
 static void
-tiff_error_no_litiff(char *func)
+tiff_error_no_litiff(const char *func)
 {
   error(EXIT_FAILURE, 0, "%s: libtiff was not found during the "
         "configuration of %s on this system. To read from TIFF files, "
@@ -151,7 +154,7 @@ gal_tiff_dir_string_read(char *string)
 
 
 /*************************************************************
- **************        Read a JPEG image        **************
+ **************        Read a TIFF image        **************
  *************************************************************/
 #ifdef HAVE_LIBTIFF
 static void
@@ -627,3 +630,153 @@ gal_tiff_read(char *filename, size_t dir, size_t minmapsize, int quietmmap)
   return NULL;
 #endif  /* HAVE_LIBTIFF */
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+/*************************************************************
+ ****************    Write into a TIFF file   ****************
+ *************************************************************/
+#ifdef HAVE_LIBTIFF
+static void
+tiff_img_write(TIFF *tif, gal_data_t *in, char *filename)
+{
+  gal_data_t *channel = in;
+  size_t index, offset, row, col, ch;
+  size_t numch = gal_list_data_number(in);
+  unsigned char *image, *buf, *colors[4] = {NULL};
+  size_t width = in->dsize[1], height = in->dsize[0];
+
+  /* Small sanity checks. */
+  if(numch==2 || numch>4)
+    error(EXIT_FAILURE, 0, "%s: only 1, 3, and 4 color channels are "
+          "acceptable, input is a list of %zu data sets", __func__,
+          numch);
+  if(in->type!=GAL_TYPE_UINT8)
+    error(EXIT_FAILURE, 0, "%s: input has a '%s' type, but TIFF "
+          "images can currently only have a 'uint8' type", __func__,
+          gal_type_name(in->type, 1));
+
+  /* Allocate memory for image */
+  image = (unsigned char*)malloc(in->size * numch);
+  if(image == NULL)
+    error(EXIT_FAILURE, errno, "%s: %s: failed to allocate memory"
+          "for image", __func__, filename);
+
+  /* Extract color channels from input data */
+  for(ch = 0; ch < numch; ch++)
+    {
+      if(channel != NULL && channel->array != NULL)
+        {
+          colors[ch] = channel->array;
+          channel = channel->next;
+        }
+      else
+        {
+          error(EXIT_FAILURE, errno, "%s: %s: missing or invalid color"
+                "channel", __func__, filename);
+          free(image);
+        }
+    }
+
+  /* Copy input data to image buffer */
+  for(row = 0; row < height; row++)
+    {
+      for(col = 0; col < width; col++)
+        {
+          index = row * width + col;
+          offset = index * numch;
+
+          for(ch = 0; ch < numch; ch++)
+            image[offset + ch] = colors[ch][index];
+        }
+    }
+
+  /* Set TIFF tags */
+  TIFFSetField(tif, TIFFTAG_IMAGEWIDTH, width);
+  TIFFSetField(tif, TIFFTAG_IMAGELENGTH, height);
+  if(numch==1)
+    TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_MINISBLACK);
+  else
+    TIFFSetField(tif, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
+  TIFFSetField(tif, TIFFTAG_BITSPERSAMPLE, 8);
+  TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, numch);
+  TIFFSetField(tif, TIFFTAG_COMPRESSION, COMPRESSION_NONE);
+  TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
+
+  /* Allocate memory for scanline buffer */
+  errno=0;
+  buf = (unsigned char*)_TIFFmalloc(TIFFScanlineSize(tif));
+  if(buf == NULL)
+    error(EXIT_FAILURE, errno, "%s: %s: failed to allocate necessary"
+          "memory for the scanline buffer" , __func__, filename);
+
+  /* Write each scanline of the image to the TIFF file */
+  for(row = height; row > 0; row--)
+    {
+      _TIFFmemcpy(buf, &image[(row - 1) * width * numch], width * numch);
+
+      if(TIFFWriteScanline(tif, buf, height - row, 0) < 0)
+        {
+          error(EXIT_FAILURE, errno, "%s: problem in writing to %s",
+                __func__, filename);
+          _TIFFfree(buf);
+          free(image);
+        }
+    }
+
+  /* Clean up */
+  _TIFFfree(buf);
+  free(image);
+}
+#endif
+
+
+
+
+
+void
+gal_tiff_write(gal_data_t* in, char* filename)
+{
+/* The TIFF library exists and will be used. */
+#ifdef HAVE_LIBTIFF
+  TIFF* tif;
+
+  /* Make sure the input isn't NULL. */
+  if(in == NULL)
+    error(EXIT_FAILURE, 0, "%s: '%s', input data is NULL",
+          __func__, filename);
+
+  /* Open the TIFF file */
+  tif = TIFFOpen(filename, "w");
+  if(tif == NULL)
+    error(EXIT_FAILURE, errno,
+          "%s: '%s' couldn't be opened for writing", __func__, filename);
+
+  /* Write to TIFF file */
+  tiff_img_write(tif, in, filename);
+
+  /* Close file */
+  TIFFClose(tif);
+
+  /* The TIFF library didn't exist. */
+#else
+  tiff_error_no_litiff(__func__);
+#endif
+}
